@@ -24,6 +24,7 @@
 
 #include <stdlib.h>
 #include <math.h>
+#include <glib/gi18n.h>
 
 #include "tomoe-canvas.h"
 
@@ -39,6 +40,18 @@ enum {
     LAST_SIGNAL,
 };
 
+enum {
+    PROP_0,
+    PROP_TOMOE_CONTEXT,
+    PROP_LOCKED,
+    PROP_WRITING,
+    PROP_AUTO_FIND_TIME,
+    PROP_HANDWRITING_LINE_COLOR,
+    PROP_ADJUSTED_LINE_COLOR,
+    PROP_ANNOTATION_COLOR,
+    PROP_AXIS_COLOR
+};
+
 typedef struct _TomoeCanvasPriv TomoeCanvasPriv;
 struct _TomoeCanvasPriv
 {
@@ -46,9 +59,9 @@ struct _TomoeCanvasPriv
     gint             width;
     gint             height;
 
-    GdkGC           *handwrite_line_gc;
-    GdkGC           *adjust_line_gc;
-    GdkGC           *annotate_gc;
+    GdkGC           *handwriting_line_gc;
+    GdkGC           *adjusted_line_gc;
+    GdkGC           *annotation_gc;
     GdkGC           *axis_gc;
 
     GdkPixmap       *pixmap;
@@ -66,8 +79,18 @@ struct _TomoeCanvasPriv
 #define TOMOE_CANVAS_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TOMOE_TYPE_CANVAS, TomoeCanvasPriv))
 
 G_DEFINE_TYPE (TomoeCanvas, tomoe_canvas, GTK_TYPE_WIDGET)
+
 /* virtual functions for GtkObject class */
 static void   dispose              (GObject           *object);
+static void   set_property         (GObject           *object,
+                                    guint              prop_id,
+                                    const GValue      *value,
+                                    GParamSpec        *pspec);
+static void   get_property         (GObject           *object,
+                                    guint              prop_id,
+                                    GValue            *value,
+                                    GParamSpec        *pspec);
+
 /* virtual functions for GtkWidget class */
 static void   realize              (GtkWidget         *widget);
 static void   size_allocate        (GtkWidget         *widget,
@@ -95,6 +118,7 @@ static void   tomoe_canvas_draw_line            (TomoeCanvas       *canvas,
 static void   tomoe_canvas_draw_background      (TomoeCanvas       *canvas,
                                                  gboolean           draw);
 static void   tomoe_canvas_draw_axis            (TomoeCanvas       *canvas);
+static void   tomoe_canvas_refresh              (TomoeCanvas       *canvas);
 static void   tomoe_canvas_resize_writing       (TomoeCanvas       *canvas,
                                                  gdouble            x_rate,
                                                  gdouble            y_rate);
@@ -106,7 +130,7 @@ static void   _init_gc                          (TomoeCanvas       *canvas);
 static void   draw_stroke                       (GList             *points,
                                                  TomoeCanvas       *canvas,
                                                  guint              index);
-static void   draw_annotate                     (GList             *points,
+static void   draw_annotation                   (GList             *points,
                                                  TomoeCanvas       *canvas,
                                                  guint              index);
 
@@ -124,6 +148,23 @@ tomoe_canvas_class_init (TomoeCanvasClass *klass)
 {
     GObjectClass   *gobject_class = G_OBJECT_CLASS (klass);
     GtkWidgetClass *widget_class  = GTK_WIDGET_CLASS (klass);
+    GParamSpec *spec;
+
+    gobject_class->dispose             = dispose;
+    gobject_class->set_property        = set_property;
+    gobject_class->get_property        = get_property;
+    widget_class->realize              = realize;
+    widget_class->size_allocate        = size_allocate;
+    widget_class->expose_event         = expose_event;
+    widget_class->button_press_event   = button_press_event;
+    widget_class->button_release_event = button_release_event;
+    widget_class->motion_notify_event  = motion_notify_event;
+
+    klass->find                        = tomoe_canvas_real_find;
+    klass->clear                       = tomoe_canvas_real_clear;
+    klass->normalize                   = tomoe_canvas_real_normalize;
+    klass->stroke_added                = NULL;
+    klass->stroke_reverted             = NULL;
 
     canvas_signals[FIND_SIGNAL] =
       g_signal_new ("find",
@@ -145,7 +186,7 @@ tomoe_canvas_class_init (TomoeCanvasClass *klass)
       g_signal_new ("normalize",
   		  G_TYPE_FROM_CLASS (klass),
   		  G_SIGNAL_RUN_LAST,
-  		  G_STRUCT_OFFSET (TomoeCanvasClass, normalize),
+ 		  G_STRUCT_OFFSET (TomoeCanvasClass, normalize),
   		  NULL, NULL,
   		  g_cclosure_marshal_VOID__VOID,
   		  G_TYPE_NONE, 0);
@@ -166,19 +207,75 @@ tomoe_canvas_class_init (TomoeCanvasClass *klass)
   		  g_cclosure_marshal_VOID__VOID,
   		  G_TYPE_NONE, 0);
 
-    gobject_class->dispose             = dispose;
-    widget_class->realize              = realize;
-    widget_class->size_allocate        = size_allocate;
-    widget_class->expose_event         = expose_event;
-    widget_class->button_press_event   = button_press_event;
-    widget_class->button_release_event = button_release_event;
-    widget_class->motion_notify_event  = motion_notify_event;
+    spec = g_param_spec_object (
+        "tomoe-context",
+        N_("Tomoe context"),
+        N_("A TomoeContext which stores handwriting dictionaries. "
+           "TomoeCanvas doesn't always require TomoeContext. "
+           "For example it isn't needed on viewer mode."),
+        TOMOE_TYPE_CONTEXT,
+        G_PARAM_WRITABLE);
+    g_object_class_install_property (gobject_class, PROP_TOMOE_CONTEXT, spec);
 
-    klass->find                        = tomoe_canvas_real_find;
-    klass->clear                       = tomoe_canvas_real_clear;
-    klass->normalize                   = tomoe_canvas_real_normalize;
-    klass->stroke_added                = NULL;
-    klass->stroke_reverted             = NULL;
+    spec = g_param_spec_boolean (
+        "locked",
+        N_("Locked"),
+        N_("Whether the canvas is locked drawing or not."),
+        FALSE,
+        G_PARAM_READABLE | G_PARAM_WRITABLE);
+    g_object_class_install_property (gobject_class, PROP_LOCKED, spec);
+
+    spec = g_param_spec_object (
+        "writing",
+        N_("Tomoe writing"),
+        N_("Strokes of a character to show on this canvas."),
+        TOMOE_TYPE_WRITING,
+        G_PARAM_READABLE | G_PARAM_WRITABLE);
+    g_object_class_install_property (gobject_class, PROP_WRITING, spec);
+
+    spec = g_param_spec_int (
+        "auto-find-time",
+        N_("Auto find time"),
+        N_("Delay time from releasing mouse button to starting auto finding. "
+           "Set 0 to find immediatly. Set -1 to disable auto finding."),
+        -1, G_MAXINT32, 0,
+        G_PARAM_READABLE | G_PARAM_WRITABLE);
+    g_object_class_install_property (gobject_class, PROP_AUTO_FIND_TIME, spec);
+
+    spec = g_param_spec_boxed (
+        "handwriting-line-color",
+        N_("Handwriting line color"),
+        N_("The color of handwriting lines."),
+        GDK_TYPE_COLOR,
+        /*G_PARAM_READABLE |*/ G_PARAM_WRITABLE);
+    g_object_class_install_property (gobject_class,
+                                     PROP_HANDWRITING_LINE_COLOR, spec);
+
+    spec = g_param_spec_boxed (
+        "adjusted-line-color",
+        N_("Adjusted line color"),
+        N_("The color of adjusted handwriting lines."),
+        GDK_TYPE_COLOR,
+        /*G_PARAM_READABLE |*/ G_PARAM_WRITABLE);
+    g_object_class_install_property (gobject_class,
+                                     PROP_ADJUSTED_LINE_COLOR, spec);
+
+    spec = g_param_spec_boxed (
+        "annotation-color",
+        N_("Annotation color"),
+        N_("The color of annotation strings."),
+        GDK_TYPE_COLOR,
+        /*G_PARAM_READABLE |*/ G_PARAM_WRITABLE);
+    g_object_class_install_property (gobject_class,
+                                     PROP_ANNOTATION_COLOR, spec);
+
+    spec = g_param_spec_boxed (
+        "axis-color",
+        N_("Axis color"),
+        N_("The color of axis lines."),
+        GDK_TYPE_COLOR,
+        /*G_PARAM_READABLE |*/ G_PARAM_WRITABLE);
+    g_object_class_install_property (gobject_class, PROP_AXIS_COLOR, spec);
 
     g_type_class_add_private (gobject_class, sizeof(TomoeCanvasPriv));
 }
@@ -188,25 +285,24 @@ tomoe_canvas_init (TomoeCanvas *canvas)
 {
     TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
     
-    priv->size              = TOMOE_CANVAS_DEFAULT_SIZE;
-    priv->width             = TOMOE_CANVAS_DEFAULT_SIZE;
-    priv->height            = TOMOE_CANVAS_DEFAULT_SIZE;
+    priv->width               = TOMOE_CANVAS_DEFAULT_SIZE;
+    priv->height              = TOMOE_CANVAS_DEFAULT_SIZE;
 
-    priv->handwrite_line_gc = NULL;
-    priv->adjust_line_gc    = NULL;
-    priv->annotate_gc       = NULL;
-    priv->axis_gc           = NULL;
+    priv->handwriting_line_gc = NULL;
+    priv->adjusted_line_gc    = NULL;
+    priv->annotation_gc       = NULL;
+    priv->axis_gc             = NULL;
 
-    priv->pixmap            = NULL;
-    priv->drawing           = FALSE;
+    priv->pixmap              = NULL;
+    priv->drawing             = FALSE;
 
-    priv->context           = NULL;
-    priv->writing           = tomoe_writing_new ();
-    priv->candidates        = NULL;
+    priv->context             = NULL;
+    priv->writing             = tomoe_writing_new ();
+    priv->candidates          = NULL;
 
-    priv->auto_find_time    = 0;
-    priv->auto_find_id      = 0;
-    priv->locked            = FALSE;
+    priv->auto_find_time      = 0;
+    priv->auto_find_id        = 0;
+    priv->locked              = FALSE;
 
     instance_list = g_list_append (instance_list, (gpointer) canvas);
 }
@@ -219,19 +315,19 @@ dispose (GObject *object)
 
     instance_list = g_list_remove (instance_list, (gpointer) canvas);
 
-    if (priv->handwrite_line_gc) {
-        g_object_unref (priv->handwrite_line_gc);
-        priv->handwrite_line_gc = NULL;
+    if (priv->handwriting_line_gc) {
+        g_object_unref (priv->handwriting_line_gc);
+        priv->handwriting_line_gc = NULL;
     }
 
-    if (priv->adjust_line_gc) {
-        g_object_unref (priv->adjust_line_gc);
-        priv->adjust_line_gc = NULL;
+    if (priv->adjusted_line_gc) {
+        g_object_unref (priv->adjusted_line_gc);
+        priv->adjusted_line_gc = NULL;
     }
 
-    if (priv->annotate_gc) {
-        g_object_unref (priv->annotate_gc);
-        priv->annotate_gc = NULL;
+    if (priv->annotation_gc) {
+        g_object_unref (priv->annotation_gc);
+        priv->annotation_gc = NULL;
     }
 
     if (priv->axis_gc) {
@@ -249,6 +345,11 @@ dispose (GObject *object)
         priv->candidates     = NULL;
     }
 
+    if (priv->context) {
+        g_object_unref (priv->context);
+        priv->context = NULL;
+    }
+
     if (priv->writing) {
         g_object_unref (priv->writing);
         priv->writing = NULL;
@@ -263,10 +364,120 @@ dispose (GObject *object)
         G_OBJECT_CLASS(tomoe_canvas_parent_class)->dispose(object);
 }
 
+static void
+set_property (GObject *object,
+              guint prop_id,
+              const GValue *value,
+              GParamSpec *pspec)
+{
+    TomoeCanvas *canvas = TOMOE_CANVAS (object);
+
+    switch (prop_id) {
+    case PROP_TOMOE_CONTEXT:
+    {
+        TomoeContext *ctx = TOMOE_CONTEXT (g_value_get_object (value));
+        tomoe_canvas_set_context (canvas, ctx);
+        break;
+    }
+    case PROP_LOCKED:
+    {
+        tomoe_canvas_set_locked (canvas, g_value_get_boolean (value));
+        break;
+    }
+    case PROP_WRITING:
+    {
+        TomoeWriting *writing = TOMOE_WRITING (g_value_get_object (value));
+        tomoe_canvas_set_writing (canvas, writing);
+        break;
+    }
+    case PROP_AUTO_FIND_TIME:
+    {
+        tomoe_canvas_set_auto_find_time (canvas, g_value_get_int (value));
+        break;
+    }
+    case PROP_HANDWRITING_LINE_COLOR:
+    {
+        GdkColor *color = g_value_get_boxed (value);
+        tomoe_canvas_set_handwriting_line_color (canvas, color);
+        break;
+    }
+    case PROP_ADJUSTED_LINE_COLOR:
+    {
+        GdkColor *color = g_value_get_boxed (value);
+        tomoe_canvas_set_adjusted_line_color (canvas, color);
+        break;
+    }
+    case PROP_ANNOTATION_COLOR:
+    {
+        GdkColor *color = g_value_get_boxed (value);
+        tomoe_canvas_set_annotation_color (canvas, color);
+        break;
+    }
+    case PROP_AXIS_COLOR:
+    {
+        GdkColor *color = g_value_get_boxed (value);
+        tomoe_canvas_set_axis_color (canvas, color);
+        break;
+    }
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+        break;
+    }
+}
+
+static void
+get_property (GObject *object,
+              guint prop_id,
+              GValue *value,
+              GParamSpec *pspec)
+{
+    TomoeCanvas *canvas = TOMOE_CANVAS (object);
+    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+
+    switch (prop_id) {
+    case PROP_TOMOE_CONTEXT:
+        g_value_set_object (value, G_OBJECT (priv->context));
+        break;
+    case PROP_LOCKED:
+    {
+        g_value_set_boolean (value, priv->locked);
+        break;
+    }
+    case PROP_WRITING:
+    {
+        TomoeWriting *writing = tomoe_canvas_get_writing (canvas);
+        g_value_set_object (value, G_OBJECT (priv->context));
+        g_object_unref (writing);
+        break;
+    }
+    case PROP_AUTO_FIND_TIME:
+    {
+        g_value_set_int (value, priv->auto_find_time);
+        break;
+    }
+    case PROP_HANDWRITING_LINE_COLOR:
+        /* FIXME */
+        break;
+    case PROP_ADJUSTED_LINE_COLOR:
+        /* FIXME */
+        break;
+    case PROP_ANNOTATION_COLOR:
+        /* FIXME */
+        break;
+    case PROP_AXIS_COLOR:
+        /* FIXME */
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+        break;
+    }
+}
+
 GtkWidget *
 tomoe_canvas_new (void)
 {
     return GTK_WIDGET(g_object_new (TOMOE_TYPE_CANVAS,
+                                    "tomoe-context", NULL,
                                     NULL));
 }
 
@@ -296,17 +507,19 @@ realize (GtkWidget *widget)
 	    GDK_LEAVE_NOTIFY_MASK;
 
     widget->window = gdk_window_new (gtk_widget_get_parent_window (widget),
-		    &attributes,
-		    GDK_WA_X | GDK_WA_Y |
-		    GDK_WA_COLORMAP |
-		    GDK_WA_VISUAL);
+                                     &attributes,
+                                     GDK_WA_X | GDK_WA_Y |
+                                     GDK_WA_COLORMAP |
+                                     GDK_WA_VISUAL);
     gdk_window_set_user_data (widget->window, widget);
     widget->style = gtk_style_attach (widget->style, widget->window);
 
-    gdk_window_set_background (widget->window, &widget->style->bg [GTK_STATE_NORMAL]);
+    gdk_window_set_background (widget->window,
+                               &widget->style->bg [GTK_STATE_NORMAL]);
 
     font_desc = pango_font_description_from_string ("Sans 12");
     gtk_widget_modify_font (widget, font_desc);
+    pango_font_description_free (font_desc);
 }
 
 static void
@@ -314,9 +527,11 @@ size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
     TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (widget);
 
+    if (GTK_WIDGET_CLASS (tomoe_canvas_parent_class)->size_allocate)
+        GTK_WIDGET_CLASS (tomoe_canvas_parent_class)->size_allocate (widget, allocation);
+
     priv->width = allocation->width;
     priv->height = allocation->height;
-    priv->size = allocation->width;
 
     if (GTK_WIDGET_REALIZED (widget)) {
         if (priv->pixmap)
@@ -328,8 +543,6 @@ size_allocate (GtkWidget *widget, GtkAllocation *allocation)
                                       -1);
     	tomoe_canvas_refresh (TOMOE_CANVAS (widget));
     }
-    if (GTK_WIDGET_CLASS (tomoe_canvas_parent_class)->size_allocate)
-        GTK_WIDGET_CLASS (tomoe_canvas_parent_class)->size_allocate (widget, allocation);
 }
 
 static gint
@@ -366,8 +579,6 @@ button_press_event (GtkWidget *widget, GdkEventButton *event)
         priv->auto_find_id = 0;
     }
 
-    /* tomoe_canvas_refresh (canvas); */
-
     if (event->button == 1) {
         priv->drawing = TRUE;
         tomoe_writing_move_to (priv->writing, (gint) event->x, (gint) event->y);
@@ -393,8 +604,7 @@ button_release_event (GtkWidget *widget, GdkEventButton *event)
     if (priv->locked) return retval;
     if (!priv->drawing) return retval;
 
-    /* tomoe_writing_line_to (priv->writing, (gint) event->x, (gint) event->y); */
-    /* draw_annotate (points, canvas, tomoe_writing_get_n_strokes (priv->writing) + 1); */
+    /* FIXME: draw annotation */
 
     priv->drawing = FALSE;
 
@@ -484,7 +694,7 @@ tomoe_canvas_real_find (TomoeCanvas *canvas)
     for (list = strokes, i = 1; list; list = g_list_next (list), i++) {
         GList *points = (GList *) list->data;
  
-        draw_annotate (points, canvas, i);
+        draw_annotation (points, canvas, i);
     }
 
     if (priv->candidates) {
@@ -492,9 +702,10 @@ tomoe_canvas_real_find (TomoeCanvas *canvas)
         priv->candidates     = NULL;
     }
 
-    writing = _tomoe_writing_new_scale_writing (priv->writing,
-                                                (gdouble) TOMOE_WRITING_WIDTH / priv->width,
-                                                (gdouble) TOMOE_WRITING_WIDTH / priv->height);
+    writing = _tomoe_writing_new_scale_writing (
+        priv->writing,
+        (gdouble) TOMOE_WRITING_WIDTH  / priv->width,
+        (gdouble) TOMOE_WRITING_HEIGHT / priv->height);
 
     query = tomoe_query_new ();
     tomoe_query_set_writing (query, writing);
@@ -532,24 +743,8 @@ tomoe_canvas_real_clear (TomoeCanvas *canvas)
     tomoe_canvas_refresh (canvas);
 }
 
-TomoeWriting *
-tomoe_canvas_get_writing (TomoeCanvas *canvas)
-{
-    TomoeWriting *writing;
-    TomoeCanvasPriv *priv;
-
-    g_return_val_if_fail (TOMOE_IS_CANVAS (canvas), NULL);
-
-    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-
-    writing = _tomoe_writing_new_scale_writing (priv->writing,
-                                                (gdouble) TOMOE_WRITING_WIDTH / priv->width,
-                                                (gdouble) TOMOE_WRITING_WIDTH / priv->height);
-    return writing;
-}
-
 void
-tomoe_canvas_set_writing (TomoeCanvas *canvas, TomoeWriting *writing)
+tomoe_canvas_set_context (TomoeCanvas *canvas, TomoeContext *context)
 {
     TomoeCanvasPriv *priv;
 
@@ -557,19 +752,178 @@ tomoe_canvas_set_writing (TomoeCanvas *canvas, TomoeWriting *writing)
 
     priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
 
-    if (priv->writing)
-        g_object_unref (priv->writing);
-    priv->writing = writing;
+    if (priv->context)
+        g_object_unref (priv->context);
+    if (context)
+        g_object_ref (context);
+    priv->context = context;
 
-    if (GTK_WIDGET_REALIZED (GTK_WIDGET (canvas)))
-    	tomoe_canvas_refresh (canvas);
+    g_object_notify (G_OBJECT (canvas), "tomoe-context");
 }
 
 void
-tomoe_canvas_lock (TomoeCanvas *canvas, gboolean lock)
+tomoe_canvas_set_locked (TomoeCanvas *canvas, gboolean lock)
+{
+    TomoeCanvasPriv *priv;
+
+    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
+
+    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+    priv->locked = lock;
+
+    g_object_notify (G_OBJECT (canvas), "locked");
+}
+
+gboolean
+tomoe_canvas_is_locked (TomoeCanvas *canvas)
+{
+    TomoeCanvasPriv *priv;
+
+    g_return_val_if_fail (TOMOE_IS_CANVAS (canvas), FALSE);
+
+    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+    return priv->locked;
+}
+
+void
+tomoe_canvas_set_writing (TomoeCanvas *canvas, TomoeWriting *writing)
+{
+    TomoeCanvasPriv *priv;
+    TomoeWriting *new_writing = NULL;
+
+    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
+
+    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+
+    if (writing)
+        new_writing = _tomoe_writing_new_scale_writing (
+            writing,
+            (gdouble) priv->width  / TOMOE_WRITING_WIDTH,
+            (gdouble) priv->height / TOMOE_WRITING_HEIGHT);
+
+    if (priv->writing)
+        g_object_unref (priv->writing);
+    priv->writing = new_writing;
+
+    if (GTK_WIDGET_REALIZED (GTK_WIDGET (canvas)))
+    	tomoe_canvas_refresh (canvas);
+
+    g_object_notify (G_OBJECT (canvas), "writing");
+}
+
+TomoeWriting *
+tomoe_canvas_get_writing (TomoeCanvas *canvas)
+{
+    TomoeWriting *writing = NULL;
+    TomoeCanvasPriv *priv;
+
+    g_return_val_if_fail (TOMOE_IS_CANVAS (canvas), NULL);
+
+    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+
+    if (priv->writing)
+        writing = _tomoe_writing_new_scale_writing (
+            priv->writing,
+            (gdouble) TOMOE_WRITING_WIDTH  / priv->width,
+            (gdouble) TOMOE_WRITING_HEIGHT / priv->height);
+
+    return writing;
+}
+
+void
+tomoe_canvas_set_auto_find_time (TomoeCanvas *canvas, gint time_msec)
 {
     TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-    priv->locked = lock;
+
+    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
+
+    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+    if (time_msec < 0)
+        priv->auto_find_time = -1;
+    else
+        priv->auto_find_time = time_msec;
+
+    g_object_notify (G_OBJECT (canvas), "auto-find-time");
+}
+
+gint
+tomoe_canvas_get_auto_find_time (TomoeCanvas *canvas)
+{
+    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+
+    g_return_val_if_fail (TOMOE_IS_CANVAS (canvas), -1);
+
+    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+
+    return priv->auto_find_time;
+}
+
+static void
+tomoe_canvas_gc_set_foreground (GdkGC *gc, GdkColor *color)
+{
+    GdkColor default_color = { 0, 0x0000, 0x0000, 0x0000 };
+
+    if (color) {
+        gdk_colormap_alloc_color (gdk_colormap_get_system (), color,
+                                  TRUE, TRUE);
+        gdk_gc_set_foreground (gc, color);
+    } else {
+        gdk_colormap_alloc_color (gdk_colormap_get_system (), &default_color,
+                                  TRUE, TRUE);
+        gdk_gc_set_foreground (gc, &default_color);
+    }
+}
+
+void
+tomoe_canvas_set_handwriting_line_color (TomoeCanvas *canvas, GdkColor *color)
+{
+    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+
+    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
+
+    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+    tomoe_canvas_gc_set_foreground (priv->handwriting_line_gc, color);
+
+    g_object_notify (G_OBJECT (canvas), "handwriting-line-color");
+}
+
+void
+tomoe_canvas_set_adjusted_line_color (TomoeCanvas *canvas, GdkColor *color)
+{
+    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+
+    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
+
+    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+    tomoe_canvas_gc_set_foreground (priv->adjusted_line_gc, color);
+
+    g_object_notify (G_OBJECT (canvas), "adjusted-line-color");
+}
+
+void
+tomoe_canvas_set_annotation_color (TomoeCanvas *canvas, GdkColor *color)
+{
+    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+
+    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
+
+    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+    tomoe_canvas_gc_set_foreground (priv->annotation_gc, color);
+
+    g_object_notify (G_OBJECT (canvas), "annotation-color");
+}
+
+void
+tomoe_canvas_set_axis_color (TomoeCanvas *canvas, GdkColor *color)
+{
+    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+
+    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
+
+    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+    tomoe_canvas_gc_set_foreground (priv->axis_gc, color);
+
+    g_object_notify (G_OBJECT (canvas), "axis-color");
 }
 
 static void
@@ -611,14 +965,21 @@ get_char_size (TomoeCanvas *canvas, GdkRectangle *rect)
 }
 
 static void
-tomoe_canvas_resize_writing (TomoeCanvas *canvas, gdouble x_rate, gdouble y_rate)
+tomoe_canvas_resize_writing (TomoeCanvas *canvas,
+                             gdouble x_rate, gdouble y_rate)
 {
     TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
     TomoeWriting *new_writing;
 
-    new_writing = _tomoe_writing_new_scale_writing (priv->writing, x_rate, y_rate);
+    new_writing = _tomoe_writing_new_scale_writing (priv->writing,
+                                                    x_rate, y_rate);
 
-    tomoe_canvas_set_writing (canvas, new_writing);
+    if (priv->writing)
+        g_object_unref (priv->writing);
+    priv->writing = new_writing;
+
+    if (GTK_WIDGET_REALIZED (GTK_WIDGET (canvas)))
+    	tomoe_canvas_refresh (canvas);
 }
 
 static TomoeWriting *
@@ -652,7 +1013,12 @@ tomoe_canvas_move_writing (TomoeCanvas *canvas, gint dx, gint dy)
 
     new_writing = _tomoe_writing_new_move_writing (priv->writing, dx, dy);
 
-    tomoe_canvas_set_writing (canvas, new_writing);
+    if (priv->writing)
+        g_object_unref (priv->writing);
+    priv->writing = new_writing;
+
+    if (GTK_WIDGET_REALIZED (GTK_WIDGET (canvas)))
+    	tomoe_canvas_refresh (canvas);
 }
 
 static void
@@ -664,16 +1030,14 @@ tomoe_canvas_real_normalize (TomoeCanvas *canvas)
     gint dx, dy;
 
     get_char_size (canvas, &char_size);
-    x_rate = (priv->size * TOMOE_CANVAS_DEFAULT_RATE)
-           / char_size.width;
-    y_rate = (priv->size * TOMOE_CANVAS_DEFAULT_RATE)
-           / char_size.height;
+    x_rate = (priv->width  * TOMOE_CANVAS_DEFAULT_RATE) / char_size.width;
+    y_rate = (priv->height * TOMOE_CANVAS_DEFAULT_RATE) / char_size.height;
 
     tomoe_canvas_resize_writing (canvas, x_rate, y_rate);
 
     get_char_size (canvas, &char_size);
-    dx = ((priv->size - char_size.width)  / 2) - char_size.x;
-    dy = ((priv->size - char_size.height) / 2) - char_size.y;
+    dx = ((priv->width  - char_size.width)  / 2) - char_size.x;
+    dy = ((priv->height - char_size.height) / 2) - char_size.y;
 
     tomoe_canvas_move_writing (canvas, dx, dy);
 
@@ -689,7 +1053,7 @@ tomoe_canvas_find (TomoeCanvas *canvas)
 }
 
 guint
-tomoe_canvas_get_number_of_candidates (TomoeCanvas *canvas)
+tomoe_canvas_get_n_candidates (TomoeCanvas *canvas)
 {
     TomoeCanvasPriv *priv;
 
@@ -718,69 +1082,37 @@ tomoe_canvas_get_nth_candidate (TomoeCanvas *canvas, guint nth)
     }
 }
 
-void
-tomoe_canvas_refresh (TomoeCanvas *canvas)
+const GList *
+tomoe_canvas_get_candidates (TomoeCanvas *canvas)
 {
-    TomoeCanvasPriv *priv;
-    GtkWidget *widget;
-    guint i;
-    const GList *strokes, *list;
+    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
 
-    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
-
-    _init_gc (canvas);
-
-    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-    widget = GTK_WIDGET (canvas);
-
-    gdk_draw_rectangle (priv->pixmap,
-                        widget->style->white_gc,
-                        TRUE,
-                        0, 0,
-                        widget->allocation.width,
-                        widget->allocation.height);
-
-    tomoe_canvas_draw_axis (canvas);
-
-    if (priv->writing) {
-        strokes = tomoe_writing_get_strokes (priv->writing);
-        for (list = strokes, i = 1; list; list = g_list_next (list), i++) {
-            GList *points = (GList *) list->data;
-            draw_stroke (points, canvas, i);
-        }
-    }
-
-    gdk_draw_drawable(widget->window,
-                      widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
-                      priv->pixmap,
-                      0, 0,
-                      0, 0,
-                      widget->allocation.width, widget->allocation.height);
+    return priv->candidates;
 }
 
 void
-tomoe_canvas_revert (TomoeCanvas *canvas)
+tomoe_canvas_revert_stroke (TomoeCanvas *canvas)
 {
     TomoeCanvasPriv *priv;
-    gboolean reverted = TRUE;
+    gint n;
 
     g_return_if_fail (TOMOE_IS_CANVAS (canvas));
 
     priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
 
+    n = tomoe_writing_get_n_strokes (priv->writing);
+    if (n <= 0) return;
+
     /* remove last line */
     tomoe_writing_remove_last_stroke (priv->writing);
-    /* reverted = tomoe_writing_remove_last_stroke (priv->writing); */
-    if (reverted) {
-        tomoe_canvas_refresh (canvas);
+    tomoe_canvas_refresh (canvas);
 
-        g_signal_emit (G_OBJECT (canvas),
-                       canvas_signals[STROKE_REVERTED_SIGNAL], 0);
+    /* emit signal */
+    g_signal_emit (G_OBJECT (canvas),
+                   canvas_signals[STROKE_REVERTED_SIGNAL], 0);
+    if (tomoe_writing_get_n_strokes (priv->writing) == 0)
+        g_signal_emit (G_OBJECT (canvas), canvas_signals[CLEAR_SIGNAL], 0);
 
-        if (tomoe_writing_get_n_strokes (priv->writing) == 0)
-            g_signal_emit (G_OBJECT (canvas), canvas_signals[CLEAR_SIGNAL], 0);
-
-    }
 }
 
 void
@@ -807,118 +1139,6 @@ tomoe_canvas_has_stroke (TomoeCanvas *canvas)
     return (tomoe_writing_get_n_strokes (priv->writing) > 0);
 }
 
-void
-tomoe_canvas_set_size (TomoeCanvas *canvas, guint size)
-{
-    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-
-    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
-
-    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-    priv->width = size;
-    priv->height = size;
-
-    gtk_widget_set_size_request (GTK_WIDGET (canvas), size, size);
-    gtk_widget_queue_resize (GTK_WIDGET (canvas));
-    if (GTK_WIDGET_REALIZED (GTK_WIDGET (canvas)))
-        tomoe_canvas_refresh (canvas);
-}
-
-static void
-tomoe_canvas_gc_set_foreground (GdkGC *gc, GdkColor *color)
-{
-    GdkColor default_color = { 0, 0x0000, 0x0000, 0x0000 };
-
-    if (color) {
-        gdk_colormap_alloc_color (gdk_colormap_get_system (), color,
-                                  TRUE, TRUE);
-        gdk_gc_set_foreground (gc, color);
-    } else {
-        gdk_colormap_alloc_color (gdk_colormap_get_system (), &default_color,
-                                  TRUE, TRUE);
-        gdk_gc_set_foreground (gc, &default_color);
-    }
-}
-
-void
-tomoe_canvas_set_handwrite_line_color (TomoeCanvas *canvas, GdkColor *color)
-{
-    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-
-    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
-
-    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-    tomoe_canvas_gc_set_foreground (priv->handwrite_line_gc, color);
-}
-
-void
-tomoe_canvas_set_adjust_line_color (TomoeCanvas *canvas, GdkColor *color)
-{
-    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-
-    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
-
-    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-    tomoe_canvas_gc_set_foreground (priv->adjust_line_gc, color);
-}
-
-void
-tomoe_canvas_set_annotate_color (TomoeCanvas *canvas, GdkColor *color)
-{
-    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-
-    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
-
-    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-    tomoe_canvas_gc_set_foreground (priv->annotate_gc, color);
-}
-
-void
-tomoe_canvas_set_axis_color (TomoeCanvas *canvas, GdkColor *color)
-{
-    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-
-    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
-
-    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-    tomoe_canvas_gc_set_foreground (priv->axis_gc, color);
-}
-
-void
-tomoe_canvas_set_auto_find_time (TomoeCanvas *canvas, gint time_msec)
-{
-    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-
-    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
-
-    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-    if (time_msec < 0)
-        priv->auto_find_time = -1;
-    else
-        priv->auto_find_time = time_msec;
-}
-
-gint
-tomoe_canvas_get_auto_find_time (TomoeCanvas *canvas)
-{
-    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-
-    g_return_val_if_fail (TOMOE_IS_CANVAS (canvas), -1);
-
-    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-
-    return priv->auto_find_time;
-}
-
-void
-tomoe_canvas_set_context (TomoeCanvas *canvas, TomoeContext *context)
-{
-    TomoeCanvasPriv *priv;
-    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
-    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-    priv->context = context;
-}
-
 static void
 tomoe_canvas_append_point (TomoeCanvas *canvas, gint x, gint y)
 {
@@ -943,7 +1163,7 @@ tomoe_canvas_append_point (TomoeCanvas *canvas, gint x, gint y)
 
     _init_gc (canvas);
     tomoe_canvas_draw_line (canvas, &pp, &p,
-                            priv->handwrite_line_gc,
+                            priv->handwriting_line_gc,
                             TRUE);
 
     tomoe_writing_line_to (priv->writing, x, y);
@@ -1021,11 +1241,51 @@ tomoe_canvas_draw_axis (TomoeCanvas *canvas)
     }
 
     gdk_draw_line (priv->pixmap, priv->axis_gc,
-                   (priv->size / 2), 0,
-                   (priv->size / 2), priv->size);
+                   (priv->width / 2), 0,
+                   (priv->width / 2), priv->height);
     gdk_draw_line (priv->pixmap, priv->axis_gc,
-                   0,          (priv->size / 2),
-                   priv->size, (priv->size / 2));
+                   0,          (priv->height / 2),
+                   priv->width, (priv->height / 2));
+}
+
+static void
+tomoe_canvas_refresh (TomoeCanvas *canvas)
+{
+    TomoeCanvasPriv *priv;
+    GtkWidget *widget;
+    guint i;
+    const GList *strokes, *list;
+
+    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
+
+    _init_gc (canvas);
+
+    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+    widget = GTK_WIDGET (canvas);
+
+    gdk_draw_rectangle (priv->pixmap,
+                        widget->style->white_gc,
+                        TRUE,
+                        0, 0,
+                        widget->allocation.width,
+                        widget->allocation.height);
+
+    tomoe_canvas_draw_axis (canvas);
+
+    if (priv->writing) {
+        strokes = tomoe_writing_get_strokes (priv->writing);
+        for (list = strokes, i = 1; list; list = g_list_next (list), i++) {
+            GList *points = (GList *) list->data;
+            draw_stroke (points, canvas, i);
+        }
+    }
+
+    gdk_draw_drawable(widget->window,
+                      widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+                      priv->pixmap,
+                      0, 0,
+                      0, 0,
+                      widget->allocation.width, widget->allocation.height);
 }
 
 static void
@@ -1034,30 +1294,30 @@ _init_gc (TomoeCanvas *canvas)
     TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
     GtkWidget *widget = GTK_WIDGET (canvas);
 
-    if (!priv->adjust_line_gc) {
+    if (!priv->adjusted_line_gc) {
         GdkColor color = { 0, 0x8000, 0x0000, 0x0000 };
-        priv->adjust_line_gc = gdk_gc_new (widget->window);
-        tomoe_canvas_set_adjust_line_color (canvas, &color);
-        gdk_gc_set_line_attributes (priv->adjust_line_gc, 1,
+        priv->adjusted_line_gc = gdk_gc_new (widget->window);
+        tomoe_canvas_set_adjusted_line_color (canvas, &color);
+        gdk_gc_set_line_attributes (priv->adjusted_line_gc, 1,
                                     GDK_LINE_SOLID,
                                     GDK_CAP_BUTT,
                                     GDK_JOIN_BEVEL);
     }
 
-    if (!priv->handwrite_line_gc) {
+    if (!priv->handwriting_line_gc) {
         GdkColor color = { 0, 0x0000, 0x0000, 0x0000 };
-        priv->handwrite_line_gc = gdk_gc_new (widget->window);
-        tomoe_canvas_set_handwrite_line_color (canvas, &color);
-        gdk_gc_set_line_attributes (priv->handwrite_line_gc, 4,
+        priv->handwriting_line_gc = gdk_gc_new (widget->window);
+        tomoe_canvas_set_handwriting_line_color (canvas, &color);
+        gdk_gc_set_line_attributes (priv->handwriting_line_gc, 4,
                                     GDK_LINE_SOLID,
                                     GDK_CAP_ROUND,
                                     GDK_JOIN_ROUND);
     }
 
-    if (!priv->annotate_gc) {
+    if (!priv->annotation_gc) {
         GdkColor color = { 0, 0x8000, 0x0000, 0x0000 };
-        priv->annotate_gc = gdk_gc_new (widget->window);
-        tomoe_canvas_set_annotate_color (canvas, &color);
+        priv->annotation_gc = gdk_gc_new (widget->window);
+        tomoe_canvas_set_annotation_color (canvas, &color);
     }
 }
 
@@ -1079,14 +1339,14 @@ draw_stroke (GList *points, TomoeCanvas *canvas, guint index)
         p2 = (TomoePoint *) next->data;
 
         tomoe_canvas_draw_line (canvas, p1, p2,
-                                priv->handwrite_line_gc,
+                                priv->handwriting_line_gc,
                                 FALSE);
     }
-    draw_annotate (points, canvas, index);
+    draw_annotation (points, canvas, index);
 }
 
 static void
-draw_annotate (GList *points, TomoeCanvas *canvas, guint index)
+draw_annotation (GList *points, TomoeCanvas *canvas, guint index)
 {
     TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
     GtkWidget *widget = GTK_WIDGET (canvas);
@@ -1123,19 +1383,12 @@ draw_annotate (GList *points, TomoeCanvas *canvas, guint index)
     y += (0.5 + (0.5 * r * dy / dl) - (sign * 0.5 * r * dx / dl) - (height / 2));
 
     gdk_draw_layout (priv->pixmap,
-                     priv->annotate_gc,
+                     priv->annotation_gc,
                      x, y, layout);
 
     g_free (buffer);
     g_object_unref (layout);
 }
-const GList *
-tomoe_canvas_get_candidates (TomoeCanvas *canvas)
-{
-    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
-
-    return priv->candidates;
-}
 
 /*
  * vi:ts=4:nowrap:ai:expandtab
