@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
  *  Copyright (C) 2005 Takuro Ashie
  *
@@ -27,6 +27,9 @@
 #include "tomoe-canvas.h"
 #include "tomoe-scrollable.h"
 
+#define DEFAULT_FONT_SCALE 2
+/*#define DEFAULT_FONT_SCALE PANGO_SCALE_XX_LARGE*/
+
 enum {
     SELECTED_SIGNAL,
     LAST_SIGNAL
@@ -168,6 +171,9 @@ static void
 tomoe_char_table_init (TomoeCharTable *view)
 {
     TomoeCharTablePriv *priv = TOMOE_CHAR_TABLE_GET_PRIVATE (view);
+    GtkWidget *widget = GTK_WIDGET (view);
+    PangoFontDescription *font_desc;
+    gint size;
 
     priv->layout      = TOMOE_CHAR_TABLE_LAYOUT_SINGLE_HORIZONTAL;
     priv->canvas      = NULL;
@@ -178,6 +184,13 @@ tomoe_char_table_init (TomoeCharTable *view)
     priv->layout_list = NULL;
     priv->h_adj       = NULL;
     priv->v_adj       = NULL;
+
+    /* Set default font with large scale */
+    font_desc = pango_font_description_copy (widget->style->font_desc);
+    size = pango_font_description_get_size(font_desc);
+    pango_font_description_set_size(font_desc, size * DEFAULT_FONT_SCALE);
+    gtk_widget_modify_font (widget, font_desc);
+    pango_font_description_free (font_desc);
 }
 
 static void
@@ -245,8 +258,6 @@ static void
 realize (GtkWidget *widget)
 {
     GdkWindowAttr attributes;
-    PangoFontDescription *font_desc;
-    gint size;
 
     GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
 
@@ -276,13 +287,6 @@ realize (GtkWidget *widget)
     widget->style = gtk_style_attach (widget->style, widget->window);
 
     gdk_window_set_background (widget->window, &widget->style->bg [GTK_STATE_NORMAL]);
-
-    /* Set default font with large scale */
-    font_desc = pango_font_description_copy (widget->style->font_desc);
-    size = pango_font_description_get_size(font_desc);
-    pango_font_description_set_size(font_desc, size * PANGO_SCALE_XX_LARGE);
-    gtk_widget_modify_font (widget, font_desc);
-    pango_font_description_free (font_desc);
 }
 
 static void
@@ -433,12 +437,13 @@ expose_event (GtkWidget *widget, GdkEventExpose *event)
     if (GTK_WIDGET_CLASS(tomoe_char_table_parent_class)->expose_event)
         retval = GTK_WIDGET_CLASS(tomoe_char_table_parent_class)->expose_event (widget, event);
  
-    gdk_draw_drawable(widget->window,
-                      widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
-                      priv->pixmap,
-                      event->area.x, event->area.y,
-                      event->area.x, event->area.y,
-                      event->area.width, event->area.height);
+    if (priv->pixmap)
+        gdk_draw_drawable(widget->window,
+                          widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+                          priv->pixmap,
+                          event->area.x, event->area.y,
+                          event->area.x, event->area.y,
+                          event->area.width, event->area.height);
 
     return retval;
 }
@@ -524,6 +529,8 @@ tomoe_char_table_draw (TomoeCharTable *view)
     guint i;
     GList *node;
 
+    if (!priv->pixmap) return;
+
     get_char_frame_size (view,
                          &inner_width, &inner_height,
                          &outer_width, &outer_height);
@@ -771,7 +778,7 @@ on_canvas_find (TomoeCanvas *canvas, gpointer user_data)
     priv->prelighted = -1;
 
     if (priv->canvas)
-        n_candidates = tomoe_canvas_get_number_of_candidates (priv->canvas);
+        n_candidates = tomoe_canvas_get_n_candidates (priv->canvas);
 
     for (i = 0; i < n_candidates; i++) {
         PangoLayout *layout;
