@@ -218,7 +218,7 @@ constructor (GType type, guint n_props,
 
     canvas = tomoe_canvas_new ();
     priv->canvas = canvas;
-    tomoe_canvas_lock (TOMOE_CANVAS (canvas), TRUE);
+    tomoe_canvas_set_locked (TOMOE_CANVAS (canvas), TRUE);
     gtk_container_add (GTK_CONTAINER (frame), canvas);
     gtk_widget_set_size_request (canvas, 100, 100);
     /*
