@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
  *  Copyright (C) 2005 HIGUCHI Daisuke
  *  Copyright (C) 2006 Juernjakob Harder
@@ -24,6 +24,18 @@
 #include <tomoe-canvas.h>
 #include <tomoe-char-table.h>
 #include <tomoe-window.h>
-#include <tomoe-stroke-search.h>
+#include <tomoe-handwriting.h>
+#include <tomoe-reading-search.h>
+#include <tomoe-gucharmap.h>
+
+/* editors */
+#include <tomoe-details.h>
+#include <tomoe-edit-char.h>
+#include <tomoe-edit-meta.h>
+#include <tomoe-edit-strokes.h>
 
 #endif /* __TOMOE_GTK_H_INCLUDED__ */
+
+/*
+ * vi:ts=4:nowrap:ai:expandtab
+ */
